#include "display.h"
#include "serialio.h"
#include "terminalio.h"
#include "game.h"
#include "score.h"
#include "timer0.h"
#include "ledmatrix.h"

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>

uint32_t timer;
uint8_t ms;
uint16_t s;
//////////////////////////////////////////////////////////////////////////
// TERMINAL DISPLAY HANDLING
/////////////////////////////////////////////////////////////////////////
// we invoke this whenever we want to reset what we print to the screen.
void display_ingame_status() {
	clear_terminal();
	
	//prints the score to terminal using get_score from score.h
	move_cursor(10,5);
	printf_P(PSTR("Score: "));
	update_score();
	
	// prints the number of lives the player has
	move_cursor(10,6);
	printf_P(PSTR("Lives: "));
	update_lives();
	
	// prints the number of laps the player has completed
	move_cursor(10,7);
	printf_P(PSTR("Laps complete: "));
	update_laps();
	move_cursor(27,7);
	printf_P(PSTR("(level: %d)"), get_laps()+1);
	
	// prints the current speed of the player
	move_cursor(23, 5);
	printf_P(PSTR("Speed: "));
	update_speed();
	
	// prints the timer for this lap
	move_cursor(23, 6);
	printf_P(PSTR("Lap Timer: "));
	update_timer();
	

}

void update_score() {
	// runs whenever score is updated
	move_cursor(17, 5);
	printf_P(PSTR("%5d"), get_score());
}

void update_speed() {
	// runs on speed up or down event
	move_cursor(30,5);
	printf_P(PSTR("%d0 kph"), get_speed());
}

void update_timer() {
	timer = get_stopwatch();
	s = timer/10;
	ms = timer%10;
	// runs at the start of every loop
	move_cursor(34,6);
	printf_P(PSTR("%4d.%d s"), s, ms);
}

void update_lives(void) {
	move_cursor(17, 6);
	printf_P(PSTR("%d"), get_lives());
};

void update_laps(void) {
	move_cursor(25, 7);
	printf_P(PSTR("%d"), get_laps());
}

void display_highscore(void) {
	// prints the high score header
	move_cursor(65, 4);
	printf_P(PSTR(" High Scores"));
	move_cursor(65, 5);
	printf_P(PSTR("=-----------="));
	print_scores();
}