/*
 * score.c
 *
 * Written by Peter Sutton
 */
#include "serialio.h"
#include "terminalio.h"
#include "score.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

void check_signature(void);
void wipe_eeprom_at_runtime(void);
void add_to_highscore(uint8_t);
void get_name_from_user(void);
void read_nums_from_eeprom(void);
void read_names_from_eeprom(void);
void write_nums_to_eeprom(void);
void write_names_to_eeprom(void);

uint16_t score;
// counts the number of times a car moves for scoring purposes
static uint16_t move_count = 0;

uint16_t highnums[10];
char* highnames[10];
char* inits;

void init_score(void) {
	score = 0;
}

void add_to_score(uint16_t value) {
	score += value;
}

uint16_t get_score(void) {
	return score;
}

void count_moves(void) {
	move_count++;
}

void reset_count(void) {
	move_count = 0;
}

uint16_t get_count(void) {
	return move_count;
}

void set_highscores(void) {
	check_signature();
	read_nums_from_eeprom();
}

void compare_score_to_highscore(void) {
	uint8_t i;
	//get the index that we wish to insert
	for (i = 0; i < 10; i++) {
		if (score > highnums[i]) {
			break;
		}
	}
	
	if (i == 10) {
		return;
	} else {
		add_to_highscore(i);
	}
}

void add_to_highscore(uint8_t index) {
	get_name_from_user();
	read_names_from_eeprom();
	
	//insert score and inits into index
	uint8_t j;
	uint16_t tempscores[10];
	char * tempnames[10];
	for (j = 0; j < 10; j++) {
		if (j < index) {
			tempscores[j] = highnums[j];
			tempnames[j] = highnames[j];
		} else if (j == index) {
			tempscores[j] = score;
			tempnames[j] = inits;
		} else {
			tempscores[j] = highnums[j - 1];
			tempnames[j] = highnames[j - 1];
		}
	}
	
	for (j = 0; j < 10; j++) {
		highnums[j] = tempscores[j];
		highnames[j] = tempnames[j];
	}
	write_nums_to_eeprom();
	write_names_to_eeprom();
}

void get_name_from_user(void) {
	//we get the initials from the user
	uint8_t cursor_pos = 0;
	char serial_input = -1;
	clear_serial_input_buffer();
	inits = malloc(4);
	
	//first print a victory message
	move_cursor(15,10);
	printf_P(PSTR("Congratulations! You're part of the Winners Circle!"));
	move_cursor(19,11);
	printf_P(PSTR("What initials shall we carve on eternity? "));
	show_cursor();
	
	// now we wait for the user
	while (1) {
		if (serial_input_available()) {
			serial_input = fgetc(stdin);
		}
		if (serial_input <= 'z' && serial_input >= 'a') {
			if (cursor_pos < 3) {
				printf_P(PSTR("%c"), serial_input - 32 );
				inits[cursor_pos] = serial_input - 32;
				cursor_pos++;
			}
			serial_input = -1;
			
			} else if (serial_input <= 'Z' && serial_input >= 'A') {
				if (cursor_pos < 3) {
					printf("%c", serial_input);
					inits[cursor_pos] = serial_input;
					cursor_pos++;
				}
				serial_input = -1;
			
				} else if (serial_input == 10) {
					hide_cursor();
					inits[cursor_pos] = '\0';
					break;
				}
			}
}

void allocate_memory(void) {
	for (uint8_t i = 0; i < 10; i++) {
		highnames[i] = malloc(4);
	}
}

void set_count(uint16_t save_data) {
	move_count = save_data;
}
void set_score(uint16_t save_data) {
	score = save_data;
}

void print_scores(void) {
	uint8_t cursor_y = 6;
	for(uint8_t i = 0; i < 10; i++) {
		const void* ptr = (const void*) 40 + (i * 4);
		char name[4];
		for (uint8_t k = 0; k < 3; k++) {
			name[k] = (char) eeprom_read_byte(ptr + k);
		}
		if (name[0] == '\0') {
			name[0] = 'A';
			name[1] = 'A';
			name[2] = 'A';
		}
		name[3] = '\0';
		move_cursor(66, cursor_y + i);
		printf_P(PSTR("%s"), name);
		printf_P(PSTR(" - "));
		printf_P(PSTR("%05d"), highnums[i]);

	}
}
///////////////////EEPROM SHIT //////////////////////

void check_signature(void) {
	uint8_t sig = 169;
	uint8_t eepval;
	eepval = eeprom_read_byte((uint8_t*) 200);
	if (sig != eepval) {
		wipe_eeprom_at_runtime();
		eeprom_write_byte((uint8_t*) 200, sig);
	} 
}

void wipe_eeprom_at_runtime(void) {
	uint8_t* j;
	uint8_t* end = (uint8_t*) 0xFF;
	for (j = 0; j < end; j++) {
		eeprom_update_byte( j, 0);
	}
}

void read_nums_from_eeprom(void) {
	uint16_t* i = 0;
	uint8_t index = 0;
	uint16_t* end = (uint16_t*) 36;
	
	for (i = 0; i <= end; i += 2) {
		highnums[index] = eeprom_read_word(i);
		if (highnums[index] == 0) {
			highnums[index] = 0;
		}
		index++;
	}
}


void read_names_from_eeprom(void) {
	const void* ptr;
	for (uint8_t j = 0; j < 10; j++) {
		ptr = (const void*) 40 + (j * 4);
		for (uint8_t k = 0; k < 3; k++) {
			highnames[j][k] = (char) eeprom_read_byte(ptr + k);
		}
		highnames[j][3] = '\0';
		if (highnames[j][0] == '\0') {
			highnames[j] = "AAA";
		}
		for (uint8_t k = 0; k < 3; k++) {
			if (highnames[j][k] == '\0') {
				highnames[j][k] = ' ';
			}
		}
	}
}

void write_nums_to_eeprom(void) {
	uint16_t* ptr;
	uint8_t index = 0;
	uint16_t* end = (uint16_t*) 36;
	
	for (ptr = 0; ptr <= end; ptr += 2) {
		eeprom_update_word(ptr, highnums[index]);
		index++;
	}
}

void write_names_to_eeprom(void) {
	uint8_t name_length = 4;
	void* ptr;
		
	for (uint8_t j = 0; j < 10; j++) {
		ptr = (void*) 40 + (j * name_length);
		for (uint8_t k = 0; k < 3; k++) {
			eeprom_update_byte(ptr + k, (uint8_t) highnames[j][k]);
		}
		highnames[j][3] = '\0';
		
	}
}